#!/usr/bin/env python
# Copyright 2014-2015 Canonical Limited.
#
# This file is part of charm-helpers.
#
# charm-helpers is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3 as
# published by the Free Software Foundation.
#
# charm-helpers is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with charm-helpers.  If not, see <http://www.gnu.org/licenses/>.


import unittest
import mock

try:
    from path import Path
except ImportError:
    Path = None
    relations = None
else:
    from charmhelpers.contrib.bigdata import relations


@unittest.skipUnless(Path, 'charmhelpers.contrib.bigdata requires path.py')
class TestSpecMatchingRelation(unittest.TestCase):
    def setUp(self):
        self.data = None
        self.cache = {}
        self.relation = relations.SpecMatchingRelation(
            spec='valid',
            relation_name='test',
            required_keys=['foo'],
            datastore=mock.MagicMock(),
            cache=self.cache)
        self.relation._unfiltered_data = lambda: self.data

    def test_ready(self):
        self.data = {'unit/0': {'spec': 'valid', 'foo': 'bar'}}
        self.assertTrue(self.relation.is_ready())

    def test_not_ready(self):
        self.data = {}
        self.assertFalse(self.relation.is_ready())
        self.cache.clear()
        self.data = {'unit/0': {}}
        self.assertFalse(self.relation.is_ready())
        self.cache.clear()
        self.data = {'unit/0': {'no-spec': 'valid', 'foo': 'bar'}}
        self.assertFalse(self.relation.is_ready())
        self.cache.clear()
        self.data = {'unit/0': {'spec': 'valid', 'no-foo': 'bar'}}
        self.assertFalse(self.relation.is_ready())
        self.cache.clear()
        self.data = {'unit/0': {'spec': 'invalid', 'no-foo': 'bar'}}
        self.assertFalse(self.relation.is_ready())

    def test_invalid(self):
        self.data = {'unit/0': {'spec': 'invalid', 'foo': 'bar'}}
        self.assertRaises(ValueError, self.relation.is_ready)


if __name__ == '__main__':
    unittest.main()
